//
// This file is part of the Marble Desktop Globe.
//
// This program is free software licensed under the GNU LGPL. You can
// find a copy of this license in LICENSE.txt in the top directory of
// the source code.
//
// Copyright 2010      Dennis Nienhüser <earthwings@gentoo.org>
//

#include "MapViewLayer.h"

#include "GeoPainter.h"
#include "GeoSceneDocument.h"
#include "GeoSceneHead.h"
#include "MarblePlacemarkModel.h"
#include "MarbleWidget.h"
#include "MarbleMap.h"
#include "MarbleModel.h"

#include <QtCore/QMap>
#include <QtGui/QAbstractProxyModel>
#include <QtGui/QIcon>
#include <QtGui/QItemSelectionModel>
#include <QtGui/QKeyEvent>

#include <QtGui/QMouseEvent>
#include <QtGui/QPixmap>


namespace Marble
{

class MapViewLayerPrivate
{
    template<class T>
    struct PaintRegion {
        T index;
        QRegion region;

        PaintRegion( const T &index_, const QRegion &region_ ) :
                index( index_ ), region( region_ )
        {
            // nothing to do
        }
    };

    typedef PaintRegion<QModelIndex> ModelRegion;
 //   typedef PaintRegion<int> SkeletonRegion;

public:
    MapViewLayer *q;

	qint64 m_indexID;

	GeoDataLineString m_lineString;

    QList<ModelRegion> m_instructionRegions;

    QList<ModelRegion> m_placemarks;

    QRegion m_layerRegion;

    QAbstractProxyModel *m_proxyModel;

    int m_movingIndex;

    MarbleWidget *m_marbleWidget;

    QPixmap m_targetPixmap;

    QRect m_dirtyRect;

    QPoint m_dropStopOver;

    QPoint m_dragStopOver;

    int m_dragStopOverRightIndex;

    bool m_pointSelection;

    MarblePlacemarkModel *m_placemarkModel;

    QItemSelectionModel *m_selectionModel;

    bool m_layerDirty;

    QSize m_pixmapSize;


    int m_activeMenuIndex;


    /** Constructor */
    explicit MapViewLayerPrivate( MapViewLayer *parent, MarbleWidget *widget, qint64 index );

    /** Show a context menu at the specified position */
    void showContextMenu( const QPoint &position );


    // The following methods are mostly only called at one place in the code, but often
    // Inlined to avoid the function call overhead. Having functions here is just to
    // keep the code clean

    inline void setPoly(GeoDataLineString&);

    /** Paint icons for each placemark in the placemark model */
    inline void renderPlacemarks( GeoPainter *painter );

    /** Paint waypoint polygon */
    inline void renderLayer( GeoPainter *painter );

    /** Insert via points or emit position signal, if appropriate */
    inline bool handleMouseButtonRelease( QMouseEvent *e );

    /** Select route instructions points, start dragging trip points */
    inline bool handleMouseButtonPress( QMouseEvent *e );

    /** Dragging trip points, route polygon hovering */
    inline bool handleMouseMove( QMouseEvent *e );

    /** Escape to stop selecting points */
    inline bool handleKeyEvent( QKeyEvent *e );

    /** True if the given point (screen coordinates) is among the route instruction points */
    inline bool isInfoPoint( const QPoint &point );

    /** Paint the stopover indicator pixmap at the given position. Also repaints the old position */
    inline void paintStopOver( QRect position );

    /** Removes the stopover indicator pixmap. Also repaints its old position */
    inline void clearStopOver();
};

MapViewLayerPrivate::MapViewLayerPrivate( MapViewLayer *parent, MarbleWidget *widget, qint64 index ) :
        q( parent ), m_proxyModel( 0 ), m_movingIndex( -1 ), m_marbleWidget( widget ),
        m_targetPixmap( ":/data/bitmaps/routing_pick.png" ), m_dragStopOverRightIndex( -1 ),
        m_pointSelection( false ), m_placemarkModel( 0 ), m_selectionModel( 0 ),
        m_layerDirty( false ), m_pixmapSize( 22, 22 ), m_activeMenuIndex( -1 ),
		m_indexID(index)
{
   

}

void MapViewLayerPrivate::setPoly(GeoDataLineString& gdl)
{

	m_lineString.clear();
	m_lineString << gdl;

}

void MapViewLayerPrivate::renderPlacemarks( GeoPainter *painter )
{
    m_placemarks.clear();
    painter->setPen( QColor( Qt::black ) );
    for ( int i = 0; i < m_placemarkModel->rowCount(); ++i ) {
        QModelIndex index = m_placemarkModel->index( i, 0 );
        QVariant data = index.data( MarblePlacemarkModel::CoordinateRole );
        if ( index.isValid() && !data.isNull() ) {
            GeoDataCoordinates pos = qVariantValue<GeoDataCoordinates>( data );

            QPixmap pixmap = qVariantValue<QPixmap>( index.data( Qt::DecorationRole ) );
            if ( !pixmap.isNull() && m_selectionModel->isSelected( index ) ) {
                QIcon selected = QIcon( pixmap );
                QPixmap result = selected.pixmap( m_pixmapSize, QIcon::Selected, QIcon::On );
                painter->drawPixmap( pos, result );
            } else {
                painter->drawPixmap( pos, pixmap );
            }

            QRegion region = painter->regionFromRect( pos, m_targetPixmap.width(), m_targetPixmap.height() );
            m_placemarks.push_back( ModelRegion( index, region ) );
        }
    }
}

void MapViewLayerPrivate::renderLayer( GeoPainter *painter )
{

	painter->setPen(QColor( 255, 0, 0, 255 ) );
	painter->setBrush( QColor( 99, 198, 99, 80 ) );
	painter->drawPolyline(m_lineString);
 

}

bool MapViewLayerPrivate::handleMouseButtonPress( QMouseEvent *e )
{
    if ( m_pointSelection ) {
        return e->button() == Qt::LeftButton;
    }

    if ( m_layerRegion.contains( e->pos() ) ) {
        if ( e->button() == Qt::LeftButton ) { 
            return false;
        } else if ( e->button() == Qt::RightButton ) {
           
            return true;
        } else
            return false;
    }

    if ( e->button() != Qt::LeftButton ) {
        return false;
    }

    return false;
}

bool MapViewLayerPrivate::handleMouseButtonRelease( QMouseEvent *e )
{
    if ( e->button() != Qt::LeftButton ) {
        return false;
    }

    if ( m_pointSelection ) 
	{
        if ( e->button() == Qt::LeftButton ) {
            qreal lon( 0.0 ), lat( 0.0 );
            if ( m_marbleWidget->geoCoordinates( e->pos().x(), e->pos().y(),
                                                 lon, lat, GeoDataCoordinates::Radian ) ) {
                emit q->pointSelected( GeoDataCoordinates( lon, lat ) );
                return true;
            }
        } else {
            return false;
        }
    }

    if ( m_movingIndex >= 0 ) 
	{
        m_movingIndex = -1;
        clearStopOver();
        emit q->layerDirty();
        return true;
    }

   

    return false;
}

bool MapViewLayerPrivate::handleMouseMove( QMouseEvent *e )
{
    if ( m_pointSelection ) {
        m_marbleWidget->setCursor( Qt::CrossCursor );
        return true;
    }


    qreal lon( 0.0 ), lat( 0.0 );

	
    if ( m_marbleWidget->geoCoordinates( e->pos().x(), e->pos().y(),
                                         lon, lat, GeoDataCoordinates::Radian ) ) 
	{

        if ( m_movingIndex == 2 ) 
		{
            GeoDataCoordinates moved( lon, lat );
     
			QModelIndex idx;// = m_selectionModel->currentIndex();
			QRect dirty;

		}
		else if ( isInfoPoint( e->pos() ) ) {
            clearStopOver();
			m_movingIndex=1;
            m_marbleWidget->setCursor( Qt::ArrowCursor );
        } 
		else if ( m_layerRegion.contains( e->pos() ) ) {
            m_dropStopOver = e->pos();
			m_movingIndex=-1;
		//	return false;
            m_marbleWidget->setCursor( Qt::OpenHandCursor );
        } 
		else if ( !m_dropStopOver.isNull() ) {
            clearStopOver();
        } 
        else {
			m_movingIndex=-1;
            return false;
        }

        // Update pixmap in the map (old and new position needs repaint)
        //paintStopOver( QRect( e->pos(), m_pixmapSize ) );
        return true;
    }

    return false;
}

bool MapViewLayerPrivate::handleKeyEvent( QKeyEvent *e )
{
    if ( m_pointSelection && e->key() == Qt::Key_Escape ) {
        m_pointSelection = false;
        emit q->pointSelectionAborted();
        return true;
    }

    return false;
}

bool MapViewLayerPrivate::isInfoPoint( const QPoint &point )
{

    foreach( const ModelRegion &region, m_instructionRegions ) {
        if ( region.region.contains( point ) ) {
            return true;
        }
    }

    return false;
}


void MapViewLayerPrivate::paintStopOver( QRect dirty )
{
    m_marbleWidget->repaint( m_dirtyRect );
    int dx = 1 + m_pixmapSize.width() / 2;
    int dy = 1 + m_pixmapSize.height() / 2;
    dirty.adjust( -dx, -dy, -dx, -dy );
    m_marbleWidget->repaint( dirty );
    m_dirtyRect = dirty;
}

void MapViewLayerPrivate::clearStopOver()
{
    m_dropStopOver = QPoint();
    m_dragStopOver = QPoint();
    m_marbleWidget->repaint( m_dirtyRect );
}

MapViewLayer::MapViewLayer( MarbleWidget *widget, QWidget *parent, qint64 index ) :
        QObject( parent ), d( new MapViewLayerPrivate( this, widget,index ) )
{
    widget->installEventFilter( this );
}

MapViewLayer::~MapViewLayer()
{
    delete d;
}

QStringList MapViewLayer::renderPosition() const
{
    return QStringList() << "HOVERS_ABOVE_SURFACE";
}

bool MapViewLayer::render( GeoPainter *painter, ViewportParams *viewport,
                           const QString& renderPos, GeoSceneLayer *layer )
{
    if ( d->m_marbleWidget->mapTheme()->head()->target() != "earth" ) {
        return false;
    }

    Q_UNUSED( viewport )
    Q_UNUSED( renderPos )
    Q_UNUSED( layer )

    painter->save();
    painter->autoMapQuality();

    if ( d->m_placemarkModel) {
        d->renderPlacemarks( painter );
    }

    d->renderLayer( painter );
	
    painter->restore();
    return true;
}

void MapViewLayer::slotUpdatePoly(GeoDataLineString& gdl)
{
	d->setPoly(gdl);

}

bool MapViewLayer::eventFilter( QObject *obj, QEvent *event )
{
    Q_UNUSED( obj )

    if ( event->type() == QEvent::KeyPress ) {
        QKeyEvent *e = static_cast<QKeyEvent*>( event );
        return d->handleKeyEvent( e );
    }

    if ( event->type() == QEvent::MouseButtonPress ) {
        QMouseEvent *e = static_cast<QMouseEvent*>( event );
        return d->handleMouseButtonPress( e );
    }

    if ( event->type() == QEvent::MouseButtonRelease ) {
        QMouseEvent *e = static_cast<QMouseEvent*>( event );
        return d->handleMouseButtonRelease( e );
    }

    if ( event->type() == QEvent::MouseMove ) {
        QMouseEvent *e = static_cast<QMouseEvent*>( event );
        return d->handleMouseMove( e );
    }

    return false;
}

void MapViewLayer::setModel ( MarblePlacemarkModel *model )
{
    d->m_placemarkModel = model;
}

void MapViewLayer::synchronizeWith( QAbstractProxyModel *model, QItemSelectionModel *selection )
{
    d->m_selectionModel = selection;
    d->m_proxyModel = model;
}


void MapViewLayer::setLayerDirty( bool dirty )
{
    d->m_layerDirty = dirty;
    
    d->m_marbleWidget->repaint();
}

} // namespace Marble

#include "./temp/moc_MapViewLayer.cpp"
