//
// This file is part of the Marble Desktop Globe.
//
// This program is free software licensed under the GNU LGPL. You can
// find a copy of this license in LICENSE.txt in the top directory of
// the source code.
//
// Copyright 2010      Dennis Nienhüser <earthwings@gentoo.org>
//

#ifndef MARBLE_MAPVIEWLAYER_H
#define MARBLE_MAPVIEWLAYER_H

#include "GeoDataCoordinates.h" 
#include "GeoDataLineString.h"
#include "LayerInterface.h"

#include <QtCore/QModelIndex>
#include <QtGui/QItemSelection>

class QAbstractProxyModel;

namespace Marble
{

class MarbleWidget;
class MarblePlacemarkModel;
class MapViewLayerPrivate;
class GeoDataLineString;

/**
  * @brief A paint layer that serves as a view on a route model
  */
class MapViewLayer: public QObject, public LayerInterface
{
    Q_OBJECT

public:
    /**
      * @brief Constructor
      * @param widget The marble widget used for geopos <-> screenpos transformations
      *   and repainting of (small) areas. Must not be null
      * @param parent Optional parent widget
      */
    explicit MapViewLayer( MarbleWidget *widget, QWidget *parent = 0, qint64 index=0);

    /** Destructor */
    ~MapViewLayer();

    /** Reimplemented from LayerInterface. We'll hover above the surface */
    QStringList renderPosition() const;

    /** Reimplemented from LayerInterface. Paints route items and placemarks */
    bool render( GeoPainter *painter, ViewportParams *viewport,
                 const QString &renderPos = "NONE", GeoSceneLayer *layer = 0 );

    /**
      * Set the proxy model another QAbstractItemView uses that should share
      * its selection model with us. Needed because this class uses an unfiltered
      * model which has different indices than a filtered one.
      */
    void synchronizeWith( QAbstractProxyModel *model, QItemSelectionModel *selection );

   
    /**
      * Set the placemark model to use. Implicitly removes the routing model.
      */
    void setModel ( MarblePlacemarkModel *model );

    
    /**
      * The layer is dirty (needs an update) and should be painted to indicate that
      */
    void setLayerDirty( bool dirty );

   

Q_SIGNALS:
    /**
      * A placemark was selected (clicked) by the user. The index belongs to
      * the model set via setModel
      */
    void placemarkSelected( const QModelIndex &index );

    /**
      * A point was selected by a mouse click after setPointSelectionEnabled(true) was called
      */
    void pointSelected( const GeoDataCoordinates &coordinates );

    /**
      * Selection of points was aborted by the user without selecting a point
      */
    void pointSelectionAborted();

    /**
      * The layer skeleton was changed permanently (no dragging)
      */
    void layerDirty();

public Q_SLOTS:
	void slotUpdatePoly(GeoDataLineString&);

protected:
    /** Overriding QWidget, used to make the layer interactive */
    bool eventFilter( QObject *obj, QEvent *event );


private:
    MapViewLayerPrivate *const d;
    friend class MapViewLayerPrivate;

};

} // namespace Marble

#endif
